package pt.utl.ist.online.learning.engines.info;

import java.io.Serializable;

import pt.utl.ist.online.learning.engines.OnlineEngine;

/**
 * Specifies the methods called during the training of engines used to report.
 *  This methods allow one to receive information at each stage of the training
 *   algorithms.
 * The usual usage is for printing but it can be used to save into a database etc..
 * 
 * @author Joao L. M. Pereira
 * @version 0.1
 * @since 2015-06-18
 *
 * @param <E> class of the data instances used by the OnlineEngine that will call 
 * the methods of the EngineObserver object
 */
public interface EngineObserver<E> extends Serializable{
	
	/**
	 * Passes information to this EngineObserver, this information is the 
	 * OnlineEngine object, a classifier name, the iteration number when
	 *  it was called and how many updates the data instances generated
	 * 
	 * @param engine the OnlineEngine being trained
	 * @param classifier a string containing the classifier name, used to d
	 * distinguish binary classifiers at multi-class methods
	 * @param iteration the number of iterations in the reported stage
	 * @param updates the number of updates that the data instances provoked in this step
	 * @param objectiveValue the objective function value at this stage
	 */
	void notify(OnlineEngine<E> engine, String classifier, int iteration, int updates,
			double objectiveValue);
	
	/**
	 * Passes information to this EngineObserver, this information is the 
	 * OnlineEngine object, a classifier name, the iteration number when
	 *  it was called and how many updates the data instances generated
	 * 
	 * @param engine the OnlineEngine being trained
	 * @param classifier a string containing the classifier name, used to d
	 * distinguish binary classifiers at multi-class methods
	 * @param iteration the number of iterations in the reported stage
	 * @param updates the number of updates that the data instances provoked in this step
	 */
	void notify(OnlineEngine<E> engine, String classifier, int iteration, int updates);
	
	/**
	 * Tells this OnlineObserver that the OnlineEngine identified by the 
	 * classifier string finished training
	 * 
	 * @param engine the OnlineEngine that finished training
	 * @param classifier string containing the classifier name
	 */
	void close(OnlineEngine<E> engine, String classifier);
}
