package pt.utl.ist.online.learning.kernels;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.HashMap;
import java.util.Map;

import pt.utl.ist.online.learning.exceptions.InvalidVectorIndexException;
import pt.utl.ist.online.learning.utils.DataObject;

public class ComputedKernel<E> extends Kernel<E> {
	transient private double[] computedMatrix ;
	private Kernel<E> kernel;
	transient private Map<Integer,Integer> conversorId;

	public ComputedKernel(double[] computedMatrix, Map<Integer,Integer> conversorId, Kernel<E> kernel) {
		super(kernel.getBNorm(),false,false);

		this.computedMatrix = computedMatrix;
		this.conversorId = conversorId;
		this.kernel = kernel;
	}


	public ComputedKernel(double[] computedMatrix, Kernel<E> kernel) {
		super(kernel.getBNorm(),false,false);
		this.computedMatrix = computedMatrix;
		this.kernel = kernel;

		this.conversorId = null;
	}


	public String toString(){
		return kernel.toString();
	}
	@Override
	protected double computeValue(DataObject<E> v1, DataObject<E> v2) {
		int id1 = v1.getId();
		int id2 = v2.getId();
		if(id1 < 0 || id2 < 0){
			return kernel.compute(v1, v2);
		}

		if(id1 == id2 && bNorm){
			return 1;
		}

		return computedMatrix[flatten(id1,id2)];
	}


	private int getMatrixPos(int i){
		if(conversorId != null)
			return conversorId.get(i);

		return i;
	}

	private int flatten(int k, int l) {
		int i = getMatrixPos(k);
		int j = getMatrixPos(l);

		int offset = -1;

		if(i<j){
			int temp = i;
			i = j;
			j = temp;
		}

		offset = (int)(( i * (i -1) )/ 2.0 ) + j;


		return offset;
	}


	public DataObject<E> convertVector(DataObject<E> x)
			throws InvalidVectorIndexException {
		return kernel.convertVector(x);
	}


	private void writeObject(ObjectOutputStream oos) throws IOException {
		oos.defaultWriteObject();
		oos.writeObject(kernel);

	}

	private void readObject(ObjectInputStream ois) throws IOException,
	ClassNotFoundException {
		ois.defaultReadObject();
		kernel = (Kernel<E>) ois.readObject();
	}
	
}
