package pt.utl.ist.online.learning.kernels;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import pt.utl.ist.online.learning.exceptions.InvalidVectorIndexException;
import pt.utl.ist.online.learning.utils.DataObject;

/**
 * Implements the linear kernel to be used in this framework
 * 
 * @author Goncalo Simoes
 * @author Joao L. M. Pereira
 * @version 0.1
 * @since 2015-06-18
 */
public class LinearKernel extends Kernel<Map<Long, Double>> {

	private final static Long CONSTANT_DIMENSION = (long) -1;
	
	/**
	 * Creates a linear kernel
	 */
	public LinearKernel(){
		super(false,false,false);
	}
	
	@Override
	protected double computeValue(DataObject<Map<Long, Double>> v1, DataObject<Map<Long, Double>> v2) {
		Map<Long, Double> x1 = v1.getData();
		Map<Long, Double> x2 = v2.getData();
		int x1Size = x1.size();
		int x2Size = x2.size();
		if(x2Size<x1Size){
			Map<Long,Double> temp=x1;
			x1=x2;
			x2=temp;
		}
		
		double result = 0;
		for(Entry<Long,Double> entryX1 : x1.entrySet()){
			Double valX2 = x2.get(entryX1.getKey());
			if(valX2!=null){
				result+=entryX1.getValue()*valX2;
			}
		}
		return result;
	}
	
	@Override
	public DataObject<Map<Long, Double>> convertVector(DataObject<Map<Long, Double>> x) throws InvalidVectorIndexException{
		if(x.getData().containsKey(CONSTANT_DIMENSION)){
			throw new InvalidVectorIndexException(CONSTANT_DIMENSION);
		}
		
		Map<Long,Double> result = new HashMap<Long, Double>(x.getData());
		result.put(CONSTANT_DIMENSION, 1.0);
		return new DataObject<Map<Long,Double>>(result,x.getId());
	}

}
