package pt.utl.ist.online.learning.utils;

import java.io.Serializable;
/**
 * Encapsulates an object of type E with an unique identifier number.
 * 
 * @author Goncalo Simoes
 * @author Joao L. M. Pereira
 * @version 0.1
 * @since 2015-06-18 
 * 
 * @param <E> class of the object encapsulated by this DataObject
 */

public class DataObject<E> implements Serializable, Comparable<DataObject<E>> {
	private E data;

	private int id;
	
	/**
	 * Constructs a new DataObject with an object of type E and its unique identifier number.
	 * 
	 * @param object the object of type E encapsulated by this DataObject
	 * @param id the unique identifier number of the DataObject
	 */
	public DataObject(E object, int id){
		data=object;
		this.id=id;
	}
	
	/**
	 * Gets the object of type E encapsulated by this DataObject
	 * 
	 * @return the object of type E
	 */
	public E getData(){
		return data;
	}
	
	/**
	 * Gets the unique identifier of this DataObject
	 * 
	 * @return the unique identifier value
	 */
	public int getId(){
		return id;
	}
	
	@Override
	public boolean equals(Object other){
		if(other instanceof DataObject){
			return id==((DataObject) other).id;
		}
		return false;
	}
	
	@Override
	public int hashCode(){
		return id;
	}


	@Override
	public int compareTo(DataObject<E> o) {
		int id2 = o.getId();
		return id - id2;
	}
}
